import { css } from "@linaria/core";
import { omit } from "lodash";
import { defineComponent, onMounted, reactive, ref } from "vue";
import { any } from "vue-types";
import { useEditor } from "../../..";
import { Background, Layout } from "../../../typings";

export const View = defineComponent({
  props: {
    background: any<Background>(),
    layout: any<Layout>(),
  },
  setup(props, { slots }) {
    const { store, actions, helper } = useEditor();
    const viewRef = ref<HTMLElement>();
    const state = reactive({
      compId: "",
    });
    onMounted(() => {
      const compEl = viewRef.value;
      state.compId = compEl?.getAttribute("data-id") || "";
    });

    function createStyle(): any {
      const style: any = {};
      const { textAlign, offsetY, zIndex } = props.layout || {};
      if (textAlign) {
        style.textAlign = textAlign;
      }
      if (offsetY) {
        style[`margin` + ((offsetY as number) > 0 ? "Top" : "Bottom")] =
          helper.designToNaturalSize(Math.abs(offsetY as number) * -1);
      }
      if (zIndex) {
        style["zIndex"] = zIndex;
      }
      return style;
    }

    function createContentStyle() {
      const style: any = {};
      const { background, layout } = props;
      const [w, h] = (layout?.size as number[]) || [];
      if (background?.image) {
        style["backgroundImage"] = `url(${background.image})`;
      } else if (background?.color) {
        style["backgroundColor"] = background.color;
      }
      if (w) style["width"] = helper.designToNaturalSize(w);
      if (h) style["height"] = helper.designToNaturalSize(h);
      if (layout?.offsetX) {
        style["marginLeft"] = helper.designToNaturalSize(
          layout.offsetX as number
        );
      }

      return style;
    }

    return () => {
      const isComp = state.compId;
      const isSelected = store.isEditMode && store.currCompId === state.compId;

      const bgOpts = Object.values(omit(props.background, ["image", "color"]));
      const bgClasses = bgOpts.length ? `bg-${bgOpts.join(" bg-")}` : "";

      return (
        <div
          ref={viewRef}
          class={[
            store.isEditMode && isComp ? viewStyle : "view_inside",
            store.isEditMode && isSelected && "view_selected",
            bgClasses,
          ]}
          style={createStyle()}
          onClick={
            state.compId
              ? () => {
                  if (state.compId !== store.currCompId)
                    actions.pickCurrComp(state.compId);
                }
              : undefined
          }
        >
          <div class={viewContainerStyle} style={createContentStyle()}>
            {slots.default?.()}
          </div>
        </div>
      );
    };
  },
});

const viewStyle = css`
  position: relative;
  &:hover {
    outline: 1px dashed @inf-primary-color;
  }

  > * {
    pointer-events: none;
  }

  &.view_selected {
    outline: 1px solid @inf-primary-color;

    > * {
      pointer-events: auto;
    }
    &::after {
      display: none;
    }

    .view_inside:hover {
      outline: 1px dashed @inf-primary-color;
      outline-offset: -1px;
    }
  }

  .view_inside {
    position: relative;
  }
`;

const viewContainerStyle = css`
  display: inline-block;
  width: 100%;
  height: 100%;
`;
